# Nasledujúce ukážky ilustrujú časté chyby alebo nedokonalosti v riešeniach 
# priebežného testu. V niektorých prípadoch ide skutočne o chyby, ktoré 
# spôsobia, že program nebude fungovať alebo bude fungovať inak, než by mal. 
# V iných prípadoch by kód síce fungoval správne, ale dá sa zapísať 
# jednoduchšie, prehľadnejšie, alebo efektívnejšie.

# Ukážky s opravenými chybami a pár vysvetľujúcimi komentármi nájdete 
# v súbore feedback_z_testu-opravy.py.



###########################################################################

# 1. Testy sú kamaráti, nie nepriatelia.

###########################################################################

# 2. Prečo sa nespustí doctest?

def funkce4(seznam: list):
    delitelne_tremi = []
    for i in seznam:
        i = float(i)
        if i%3 == 0:
            delitelne_tremi.append(str(int(i)))
    """
    >>> funkce4(["3", "22", "330", "27", "8", "10"])
    ['3', '330', '27']
    """
    return delitelne_tremi 

###########################################################################

# 3. Čo je špatne?

def funkce2(retezec: str) -> str:
    while retezec.startswith("_"):
        retezec = retezec[1:]
    while retezec.endswith("_"):
        retezec = retezec[:-2]  # chyba!
    return f"_{retezec}_"

print(funkce2('___Hello_World___'))

###########################################################################

# 4. Čo je špatne?

seznam = list('___Hello_World___')
for i in range(0,len(seznam)-1):  # chyba!
    if seznam[i]=='_':
        seznam[i]="*"
print(''.join(seznam))

###########################################################################

# 5. Čo je špatne?

retezec = '_Hello_World'
if retezec[0] and retezec[-1] is not '_':  # 2 chyby!
    retezec = '_' + retezec + '_'
print(retezec)

###########################################################################

# 6. Čo je špatne?

def funkce5(jmena: str) -> tuple:
    a = jmena.split(";")
    b = {}
    for i in a:
        key, value = i.split(",")
        b[key] = float(value)
    c = max(b, key=b.get)
    d = min(b, key=b.get)
    e = float(sum(b.values())) / len(b)
    f = (d, c, e)
    return f

print(funkce5("Jan,20;Martin,26;Filip,25;Petr,30;Marek,25"))

###########################################################################

# 7. Prečo nebude fungovať správne?

seznam = [1.2, 5.5, 2000, 5.5]
for x in seznam:
    poradi = seznam.index(x)
    if (x >= 1000) | (x < 0.001):  # skoro chyba
        print(f"{poradi:<6n} {x:9.3e}")
    else:
        print(f"{poradi:<6n} {x:9.3f}")
pass

# Toto už bude fungovať, ale...

i = -1
for value in seznam:
    i += 1
    if value >= 0.001 and value < 1000:  # ako to zjednodušiť?
        print(f"{i:<6} {value:>7.1f}")
    else:
        print(f"{i:<6} {value:>7.1E}")
pass

# A čo toto?

for i in range(0, len(seznam)):
    if seznam[i] > 1000 or seznam[i] < 0.001:  # chybička
        radek = "{:<6}".format(i) + " " + "{:7.1E}".format(seznam[i])
        print(radek)
    else:
        radek = "{:<6}".format(i) + " " + "{:7}".format(seznam[i]) 
        print(radek)
pass

# https://docs.quantifiedcode.com/python-anti-patterns/readability/using_an_unpythonic_loop.html?highlight=enumerate  
       
###########################################################################     

# 8. Šlo by to krajšie?

slovnik = {'green': 'zelená', 'blue': 'modrá', 'yellow': 'žlutá'}
for klic in slovnik:
    print(klic, '=', slovnik[klic])
pass

# https://docs.quantifiedcode.com/python-anti-patterns/readability/not_using_items_to_iterate_over_a_dictionary.html
      
###########################################################################    

# 9. Šlo by to krajšie? 

slovnik = {'green': 'zelená', 'blue': 'modrá', 'yellow': 'žlutá'}
klice = slovnik.keys()
hodnoty = slovnik.values()
for klic, hodnota in zip(klice, hodnoty):  # ako to zjednodušiť?
    print(klic, '=', hodnota)
pass

###########################################################################

# 10. Šlo by to krajšie?
# (Kód má zo zoznamu vybrať reťazce reprezentujúce prirodzené čísla.)

seznam = ["1", "22", "33.3", "-44", "5555.5", "66", "1000"]
new = []
for value in seznam:
    if  ("." not in value) == True and int(value) > 0:  # ako to zjednodušiť?
        new.append(value)
print(new)

###########################################################################

# 11. Funguje, ale...
# (Kód má zo zoznamu vybrať čísla menšie/rovné 1000.)

vstup = [1.2, 5.5, 2000, 5.5]
vysl = []
for x in vstup:
    if x <= 1000:
        vysl.append(x) 
print(vysl)

###########################################################################

# 12. Funguje, ale...

dvojice = ['Jan,20', 'Martin,26', 'Filip,25', 'Petr,30', 'Marek,25']
slovnik = {}
for i in range(len(dvojice)):
    klic, hodnota = dvojice[i].split(',')
    slovnik.update({klic:hodnota})  # ako to zjednodušiť?
print(slovnik)

###########################################################################

# 13. Šlo by to krajšie?
# (Kód ma vypísať posledný znak reťazca.)

retezec = 'Ahoj'
print(retezec[len(retezec) - 1])  # ako to zjednodušiť?

###########################################################################

# 14. Funguje, ale...
# (Kód má vybrať zo zoznamu reťazcov tie, ktoré reprezentujú násobky čísla 3.)

def funkce4(seznam: list):
    delitelne_tremi = []
    for i in seznam:
        i = float(i)
        if i%3 == 0:
            delitelne_tremi.append(str(int(i)))
    return delitelne_tremi 

print(funkce4(["3", "22", "330", "27", "8", "10"]))

###########################################################################

# 15. Ako formátovať výstup?

i = 1
x = 5.5

print('%-6d %9.3f' % (i, x))

print('{:<6d} {:9.3f}'.format(i, x))

print(f'{i:<6d} {x:9.3f}')

print(f"{i:<6}{' '}{x:9.3f}") 

# https://realpython.com/python-f-strings/

###########################################################################

# 16. Funguje, ale...

seznam = ['Adam', 'Elisabeth', 'Johny', 'Mark', 'Mary', 'Peter', 'Susane']
a = seznam[0]
for i in seznam[1:]:
    a = a+";"+i
print(a)

###########################################################################

# 17. Funguje, ale...

retezec = 'anakonda'
zoznam = list(retezec)  # ako to zjednodušiť?
if zoznam[0] == zoznam[-1]:
    print(retezec)
pass

###########################################################################
